@@ -7,8 +7,6 @@ import re
 import shutil
 from typing import List, Callable, Iterator, Optional, Tuple
 
-import pytest
-
 from mypy import build
 from mypy.errors import CompileError
 from mypy.options import Options
@@ -17,6 +15,7 @@ from mypy.test.config import test_temp_dir
 from mypy.test.helpers import assert_string_arrays_equal
 
 from mypyc.options import CompilerOptions
+from mypyc.analysis.ircheck import assert_func_ir_valid
 from mypyc.ir.func_ir import FuncIR
 from mypyc.errors import Errors
 from mypyc.irbuild.main import build_ir
@@ -32,7 +31,7 @@ TESTUTIL_PATH = os.path.join(test_data_prefix, 'fixtures/testutil.py')
 
 class MypycDataSuite(DataSuite):
     # Need to list no files, since this will be picked up as a suite of tests
-    files = []  # type: List[str]
+    files: List[str] = []
     data_prefix = test_data_prefix
 
 
@@ -57,12 +56,13 @@ def use_custom_builtins(builtins_path: str, testcase: DataDrivenTestCase) -> Ite
         shutil.copyfile(builtins_path, builtins)
         default_builtins = True
 
-    # Actually peform the test case.
-    yield None
-
-    if default_builtins:
-        # Clean up.
-        os.remove(builtins)
+    # Actually perform the test case.
+    try:
+        yield None
+    finally:
+        if default_builtins:
+            # Clean up.
+            os.remove(builtins)
 
 
 def perform_test(func: Callable[[DataDrivenTestCase], None],
@@ -77,7 +77,7 @@ def perform_test(func: Callable[[DataDrivenTestCase], None],
         shutil.copyfile(builtins_path, builtins)
         default_builtins = True
 
-    # Actually peform the test case.
+    # Actually perform the test case.
     func(testcase)
 
     if default_builtins:
@@ -89,7 +89,9 @@ def build_ir_for_single_file(input_lines: List[str],
                              compiler_options: Optional[CompilerOptions] = None) -> List[FuncIR]:
     program_text = '\n'.join(input_lines)
 
-    compiler_options = compiler_options or CompilerOptions()
+    # By default generate IR compatible with the earliest supported Python C API.
+    # If a test needs more recent API features, this should be overridden.
+    compiler_options = compiler_options or CompilerOptions(capi_version=(3, 5))
     options = Options()
     options.show_traceback = True
     options.use_builtins_fixtures = True
@@ -114,10 +116,11 @@ def build_ir_for_single_file(input_lines: List[str],
         Mapper({'__main__': None}),
         compiler_options, errors)
     if errors.num_errors:
-        errors.flush_errors()
-        pytest.fail('Errors while building IR')
+        raise CompileError(errors.new_messages())
 
     module = list(modules.values())[0]
+    for fn in module.functions:
+        assert_func_ir_valid(fn)
     return module.functions
 
 
@@ -131,7 +134,7 @@ def update_testcase_output(testcase: DataDrivenTestCase, output: List[str]) -> N
     # We can't rely on the test line numbers to *find* the test, since
     # we might fix multiple tests in a run. So find it by the case
     # header. Give up if there are multiple tests with the same name.
-    test_slug = '[case {}]'.format(testcase.name)
+    test_slug = f'[case {testcase.name}]'
     if data_lines.count(test_slug) != 1:
         return
     start_idx = data_lines.index(test_slug)
@@ -154,13 +157,15 @@ def assert_test_output(testcase: DataDrivenTestCase,
                        message: str,
                        expected: Optional[List[str]] = None,
                        formatted: Optional[List[str]] = None) -> None:
+    __tracebackhide__ = True
+
     expected_output = expected if expected is not None else testcase.output
     if expected_output != actual and testcase.config.getoption('--update-data', False):
         update_testcase_output(testcase, actual)
 
     assert_string_arrays_equal(
         expected_output, actual,
-        '{} ({}, line {})'.format(message, testcase.file, testcase.line))
+        f'{message} ({testcase.file}, line {testcase.line})')
 
 
 def get_func_names(expected: List[str]) -> List[str]:
@@ -200,7 +205,7 @@ def show_c(cfiles: List[List[Tuple[str, str]]]) -> None:
     heading('Generated C')
     for group in cfiles:
         for cfile, ctext in group:
-            print('== {} =='.format(cfile))
+            print(f'== {cfile} ==')
             print_with_line_numbers(ctext)
     heading('End C')
 
@@ -213,12 +218,6 @@ def fudge_dir_mtimes(dir: str, delta: int) -> None:
             os.utime(path, times=(new_mtime, new_mtime))
 
 
-def replace_native_int(text: List[str]) -> List[str]:
-    """Replace native_int with platform specific ints"""
-    int_format_str = 'int32' if IS_32_BIT_PLATFORM else 'int64'
-    return [s.replace('native_int', int_format_str) for s in text]
-
-
 def replace_word_size(text: List[str]) -> List[str]:
     """Replace WORDSIZE with platform specific word sizes"""
     result = []
@@ -233,3 +232,35 @@ def replace_word_size(text: List[str]) -> List[str]:
         else:
             result.append(line)
     return result
+
+
+def infer_ir_build_options_from_test_name(name: str) -> Optional[CompilerOptions]:
+    """Look for magic substrings in test case name to set compiler options.
+
+    Return None if the test case should be skipped (always pass).
+
+    Supported naming conventions:
+
+      *_64bit*:
+          Run test case only on 64-bit platforms
+      *_32bit*:
+          Run test caseonly on 32-bit platforms
+      *_python3_8* (or for any Python version):
+          Use Python 3.8+ C API features (default: lowest supported version)
+      *StripAssert*:
+          Don't generate code for assert statements
+    """
+    # If this is specific to some bit width, always pass if platform doesn't match.
+    if '_64bit' in name and IS_32_BIT_PLATFORM:
+        return None
+    if '_32bit' in name and not IS_32_BIT_PLATFORM:
+        return None
+    options = CompilerOptions(strip_asserts='StripAssert' in name,
+                              capi_version=(3, 5))
+    # A suffix like _python3.8 is used to set the target C API version.
+    m = re.search(r'_python([3-9]+)_([0-9]+)(_|\b)', name)
+    if m:
+        options.capi_version = (int(m.group(1)), int(m.group(2)))
+    elif '_py' in name or '_Python' in name:
+        assert False, f'Invalid _py* suffix (should be _pythonX_Y): {name}'
+    return options
